@@ -1,5 +1,6 @@
 from enum import Enum
 from typing import Optional
+
 from pydantic import BaseModel, Field, validator
 
 
@@ -15,17 +16,26 @@ class PolicyStoreDetails(BaseModel):
     - type
     - credentials
     """
-    type: PolicyStoreTypes = Field(PolicyStoreTypes.OPA, description="the type of policy store, currently only OPA is officially supported")
-    url: str = Field(..., description="the url that OPA can be found in. if localhost is the host - it means OPA is on the same hostname as OPAL client.")
-    token: Optional[str] = Field(None, description="optional access token required by the policy store")
 
-    @validator('type')
+    type: PolicyStoreTypes = Field(
+        PolicyStoreTypes.OPA,
+        description="the type of policy store, currently only OPA is officially supported",
+    )
+    url: str = Field(
+        ...,
+        description="the url that OPA can be found in. if localhost is the host - it means OPA is on the same hostname as OPAL client.",
+    )
+    token: Optional[str] = Field(
+        None, description="optional access token required by the policy store"
+    )
+
+    @validator("type")
     def force_enum(cls, v):
         if isinstance(v, str):
             return PolicyStoreTypes(v)
         if isinstance(v, PolicyStoreTypes):
             return v
-        raise ValueError(f'invalid value: {v}')
+        raise ValueError(f"invalid value: {v}")
 
     class Config:
         use_enum_values = True
